'use strict'

var React = require('react')

var UploadField = require('react-upload-field/src')
var ListView    = require('react-listview/src')
var normalize  = require('react-style-normalizer')

require('../style/index.styl')

var assign = require('object-assign')

module.exports = React.createClass({

	displayName: 'ReactUploader',

	getInitialState: function(){
		return {
			overClearIndex: -1,
			files: []
		}
	},

	getDefaultProps: function(){
		return {
			displayProperty: 'name',
			idProperty: 'id',

			tagClearTool: false,

			clearToolStyle: {
				color: 'rgb(168, 168, 168)',
				cursor: 'pointer'
			},
			clearToolOverStyle: {
				color: 'rgb(127, 124, 124)'
			},

			defaultUploadFieldStyle: {
				alignSelf: 'stretch'
			},

			defaultListStyle: {
				marginTop: 10,
				marginBottom: 10,
				border: '1px solid gray',
				flex : 1,
				alignSelf: 'stretch'
			},

			defaultListProps: {
				title: 'Files to upload',
				emptyText: 'Drag files to upload'
			},
			defaultStyle: {
				minHeight: 200,
				display: 'flex',
				alignItems: 'flex-start',
				flexFlow: 'column'
			}
		}
	},

	render: function(){

		var state = this.state
		var props = this.prepareProps(this.props, state)

		return <div {...props}>
			<UploadField {...props.uploadFieldProps}/>
			<ListView {...props.listProps} />

			<div>
				<button disabled={!this.hasFiles()}>Upload</button>
				<button disabled={!this.hasFiles()}>Clear</button>
			</div>

		</div>
	},

	prepareListData: function(props, state) {
		return (state.files || []).map(function(file, index){

			return {
				name: file.name,
				size: file.size,
				type: file.type,
				id  : file.name + '-' + file.size + '-' + file.type
			}
		})
	},

	onChangeFiles: function(files) {
		this.setState({
			files: files
		})
	},

	handleFileClear: function(fileInfo, index) {
		this.setState({
			overClearIndex: -1
		})

		var newFiles = this.state.files.filter(function(file, i){
			return i != index
		})

		this.refs.uploadField.setFiles(newFiles)
	},

	hasFiles: function() {
		return this.state.files && !!this.state.files.length
	},

	upload: function(){
		return this.refs.uploadField.upload(this.props.url, {
			files: this.state.files,
			name : this.props.name
		})
	},

	prepareProps: function(thisProps, state) {

		var props = assign({}, thisProps)

		props.style = this.prepareStyle(props, state)

		props.listProps = this.prepareListProps(props, state)
		props.uploadFieldProps = this.prepareUploadFieldProps(props, state)

		return props
	},

	prepareUploadFieldProps: function(props, state) {
		var uploadFieldProps = assign({}, props.defaultUploadFieldProps, props.uploadFieldProps)

		uploadFieldProps.tagClearTool = props.tagClearTool
		uploadFieldProps.style = assign({}, props.defaultUploadFieldStyle, uploadFieldProps.style)
		uploadFieldProps.ref = "uploadField"
		uploadFieldProps.onChange = this.onChangeFiles

		return uploadFieldProps
	},

	prepareListProps: function(props, state) {
		var listProps = assign({}, props.defaultListProps, props.listProps)

		listProps.style = this.prepareListStyle(listProps, props, state)
		listProps.idProperty = props.idProperty
		listProps.displayProperty = props.displayProperty

		listProps.data = this.prepareListData(props, state)

		var wrapStyle = normalize({
			display: 'flex',
			flexFlow: 'row'
		})

		var textStyle = normalize({
			display: 'inline-block',
			flex: 1
		})

		var clearStyle = props.clearToolStyle

		listProps.renderText = function(text, item, index){
			var clearToolStyle = clearStyle

			if (state.overClearIndex === index){
				clearToolStyle = assign({}, clearStyle, props.clearToolOverStyle)
			}

			return <div style={wrapStyle}>
				<span style={textStyle}>{text}</span>
				<span style={clearToolStyle}
						onMouseEnter={this.handleFileClearMouseEnter.bind(this, item, index)}
						onMouseLeave={this.handleFileClearMouseLeave.bind(this, item, index)}
						onClick={this.handleFileClear.bind(this, item, index)}
				>✖</span>
			</div>
		}.bind(this)

		return listProps
	},

	handleFileClearMouseEnter: function(item, index) {
		this.setState({
			overClearIndex: index
		})
	},

	handleFileClearMouseLeave: function(item, index) {
		this.setState({
			overClearIndex: -1
		})
	},

	prepareListStyle: function(listProps, props, state){
		var style = assign({}, props.defaultListStyle, listProps.style)

		return style
	},

	prepareStyle: function(props, state) {
		return normalize(assign({}, props.defaultStyle, props.style))
	}
})